/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 */
public class PreferencesController {
	private static final Logger logger = Logger
			.getLogger(PreferencesController.class);


	public static void setNomenclaturalCode(String preferredCode) {
		getPrefStore().setValue(ITaxEditorConstants.CODE_PREFERENCE, preferredCode);
	}
	
	public static void setDefaultNomenclaturalCode() {
		setNomenclaturalCode(ITaxEditorConstants.DEFAULT_CODE_PREFERENCE);
	}

	public static String getPreferredNomenclaturalCodeAsString() {
		String nameCodePreference = getNameCodePreference();
		
		if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICBN)) {
			return "International Code of Botanical Nomenclature (ICBN)";
		} else if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		}
		return null;
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		String nameCodePreference = getNameCodePreference();
				
		if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICBN)) {
			return NomenclaturalCode.ICBN;
		} else if (nameCodePreference.equals(ITaxEditorConstants.CODE_PREFERENCE_ICZN)) {
			return NomenclaturalCode.ICZN;
		}
		return null;
	}
	
	private static String getNameCodePreference() {
		return getPrefStore().getString(ITaxEditorConstants.CODE_PREFERENCE);
	}

	/**
	 * Returns a <code>Set</code> of the <code>Feature</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Feature</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static List<Feature> getPreferredFeatures() {
		
		// Initialize preferredFeatureSet as necessary
		if (GlobalController.getPreferredFeatures() == null) {
				
			GlobalController.setPreferredFeatures(new ArrayList<Feature>());
			for (Feature feature : CdmSessionDataRepository.getDefault().getFeatures()) {
			
				// If the feature is set to show, add it to preferredFeatureSet
				if (PreferencesController.getFeaturePreference(feature)) {
					GlobalController.addPreferredFeature(feature);
				}
				
			}
		}
		return GlobalController.getPreferredFeatures();
	}

	/**
	 * True if <code>Feature</code> is set to "show" in preferences.
	 * 
	 * @param feature
	 * @return
	 */
	public static boolean getFeaturePreference(Feature feature) {
		
		String preferenceKey = getPreferenceKey(feature);
		
		// If feature does not yet have a pref, set it to true
		if (!PreferencesController.getPrefStore().contains(preferenceKey)) {
			PreferencesController.getPrefStore().setDefault(preferenceKey, true);
		}
		
		return PreferencesController.getPrefStore().getBoolean(preferenceKey);
	}

	/**
	 * Set the show state of a <code>Feature</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredFeatureSet</code> to null to force it be
	 * re-populated the next time {@link getPreferredFeatures} is called.
	 * 
	 * @param feature
	 * @param show
	 */
	public static void setFeaturePreference(Feature feature, boolean show) {
		GlobalController.setPreferredFeatures(null);
		PreferencesController.getPrefStore().setValue(PreferencesController.getPreferenceKey(feature), show);
	}

	public static IPreferenceStore getPrefStore() {
		return TaxEditorPlugin.getDefault().getPreferenceStore();
	}

	/**
	 * Construct a unique key using <code>Feature</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Feature feature) {
		return ITaxEditorConstants.FEATURE_PREFERENCE 
				. concat(".")
				. concat(feature.getUuid().toString());
	}

	/**
	 * Returns a <code>Set</code> of the <code>Rank</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Rank</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static Set<Rank> getPreferredRanks() {
		
		// Initialize preferredRankSet as necessary
		if (GlobalController.getPreferredRanks() == null) {
			
			GlobalController.setPreferredRanks(new HashSet<Rank>());	
						
			SortedSet<Rank> ranks = CdmSessionDataRepository.getDefault().getRanks();
			if (ranks != null) {
			
				for (Rank rank : CdmSessionDataRepository.getDefault().getRanks()) {
					
					// If the feature is set to show, add it to preferredFeatureSet
					if (PreferencesController.getRankPreference(rank)) {
						GlobalController.addPreferredRank(rank);
					}
				}
			} // else { TODO: error message
			
		}
		return GlobalController.getPreferredRanks();
	}

	/**
	 * True if <code>Rank</code> is set to "show" in preferences.
	 * 
	 * @param rank
	 * @return
	 */
	public static boolean getRankPreference(Rank rank) {
		
		String preferenceKey = PreferencesController.getPreferenceKey(rank);
		
		// If rank does not yet have a pref, set it to true
		if (!getPrefStore().contains(preferenceKey)) {
			getPrefStore().setDefault(preferenceKey, true);
		}
		
		return getPrefStore().getBoolean(preferenceKey);
	}

	/**
	 * Set the show state of a <code>Rank</code> in the 
	 * <code>PreferenceStore</code>.
	 * <p>
	 * Also sets <code>preferredRankSet</code> to null to force it be
	 * re-populated the next time {@link getPreferredRanks} is called.
	 * 
	 * @param rank
	 * @param show
	 */	
	public static void setRankPreference(Rank rank, boolean show) {
		GlobalController.setPreferredRanks(null);
		getPrefStore().setValue(PreferencesController.getPreferenceKey(rank), show);
		
	}

	/**
	 * Construct a unique key using <code>Rank</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Rank rank) {
		return ITaxEditorConstants.RANK_PREFERENCE 
				. concat(".")
				. concat(rank.getUuid().toString());
	}

	/**
	 * Returns a new name object that conforms to the nomenclatural
	 * code specified in user preferences. 
	 * 
	 * @return
	 */
	public static NonViralName getInstanceOfPreferredNameClass() {
		NomenclaturalCode code = getPreferredNomenclaturalCode();
		
		// Check whether name code preference needs to be initialized
		if (code == null) {
			setDefaultNomenclaturalCode();
		}
		
		if (code.equals(NomenclaturalCode.ICBN)) {
			return BotanicalName.NewInstance(null);
		} else if (code.equals(NomenclaturalCode.ICZN)) {
			return ZoologicalName.NewInstance(null);			
		}
		return NonViralName.NewInstance(null);
	}

	/**
	 * Appends the code (i.e. ICBN or ICZN) specified in the user preferences
	 * to a string. The result is separated by a period: "<code>key.ICBN</code>".
	 * 
	 * @param key
	 * @return
	 */
	public static String concatCodeMessageSuffix(String key) {
		String code = getNameCodePreference();
		return key + "." + code;
	}
}
