/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.controller.PreferencesController;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.GroupedComposite;
import eu.etaxonomy.taxeditor.operations.description.AddElementOperation;
import eu.etaxonomy.taxeditor.propertysheet.description.TaxonDescriptionPropertySource;

/**
 * Composite for editing a description's label / Title Cache information.
 * 
 * @author p.ciardelli
 * @created 09.09.2008
 * @version 1.0
 */
public class DescriptionLabelComposite extends GroupedComposite {
	private static final Logger logger = Logger
			.getLogger(DescriptionLabelComposite.class);

	/**
	 * ************ FONTS ************
	 */
	public static final Font DESCRIPTION_LABEL_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.ACCEPTED_TAXON_FONT);
	
	/**
	 * ************ ICONS ************
	 */
	public static final Image DESCRIPTION_LABEL_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);	
	
	private static final String EMPTY_NAME_PROMPT = "Click to edit the label for this description";

	private TaxonDescription description;

	private IUndoContext undoContext;
	
	public DescriptionLabelComposite(Composite parent, IManagedForm managedForm, TaxonDescription description) {
		super(parent, managedForm);
		
		this.description = description;
		
		this.undoContext = EditorController.getUndoContext(taxon);
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();

		createParser();
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		setIsDraggable(false);
		setFont(DESCRIPTION_LABEL_FONT);
		setIcon(DESCRIPTION_LABEL_ICON);
		
		createMenu();
		
		initTextViewer();
	}
	
	private void initTextViewer() {
		if (textViewer != null) {
			String text = getDescription().getTitleCache();
			
			if (text.length() == 0) {
				initEmptyText();
			} else {
				textViewer.getTextWidget().setText(text);
			}
		} else {
			logger.warn("TextViewer is null.");
		}
	}
	
//	protected String getEmptyTextPrompt() {
//		return "Click to edit the label for this description";
//	}

	public TaxonDescription getDescription() {
		return description;
	}

	private void createMenu() {
		
		MenuManager featuresMenu = new MenuManager("Add description element");
		featuresMenu.setRemoveAllWhenShown(true);
		featuresMenu.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				for (final Feature feature : PreferencesController.getPreferredFeatures()) {
					String text = feature.getLabel();
					manager.add(new Action(text){
						
						public void run() {
							IUndoableOperation operation = new AddElementOperation
									(this.getText(), undoContext, taxon, description, feature);
							GlobalController.executeOperation(operation);
						}
					});
				}
			}
		});
		
		// Important: add an empty action. Otherwise, when the main menu sees that
		//	there are no actions in the submenu, it will not be shown
		featuresMenu.add(new Action() {});

		ContextMenu contextMenu = createContextMenu();
		contextMenu.addSubmenu(featuresMenu);
	}

	protected void parse(String text) {
		getDescription().setTitleCache(text);
		
		// Manually refresh the property sheet to reflect changes
		setSelection();
		
		setDirty(true);		
	}
	
	public IPropertySource getPropertySource() {
		return new TaxonDescriptionPropertySource(getDescription());
	}

	@Override
	protected Font getViewerFont() {
		return null;
	}
	
	public Object getData () {
		return getDescription();
	}
}