/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationshipType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.taxeditor.controller.EditorController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.editor.SelectTaxonDialog;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class ChangeTaxonToSynonymOperation extends AbstractEditorOperation{
	
	private static final Logger logger = Logger
			.getLogger(ChangeTaxonToSynonymOperation.class);

	private Taxon destinationTaxon;
	
	public ChangeTaxonToSynonymOperation(String label,
			IUndoContext undoContext, Taxon taxon) {
		super(label, undoContext, taxon);
	}

	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		IEditorPart editor = null;
		
		try {	
			// Prompt user "Would you like to save?" before showing dialog
			// "Cancel" cancels action - "No" does not
			editor = EditorController.getEditorByTaxon(taxon);
			if (editor.isDirty()) {
				if (!MessageDialog.openConfirm(GlobalController.getShell(), "Save before proceeding", 
						"All changes must be saved before proceeding with this action.\n\n" +
						"Press \"OK\" to save and continue, or \"Cancel\" to cancel this action.")) {
					return Status.CANCEL_STATUS;
				}
				editor.doSave(null);
			}
//			if (UiUtil.getActivePage().saveEditor(oldEditor, true) == false) {
//				return;
//			}
		} catch (PartInitException e1) {
			e1.printStackTrace();
		}
		
		// Get destination taxon from dialog
		Shell shell = GlobalController.getShell();
		SelectTaxonDialog dialog = new SelectTaxonDialog(shell, SelectTaxonDialog.TAXON_TO_SYNONYM);
		destinationTaxon = dialog.open(taxon);
		
		// Abort action if user cancelled dialog without choosing a taxon
		if (destinationTaxon == null) {
			return Status.CANCEL_STATUS;
		}
				
		
		return makeTaxonSynonym(taxon, destinationTaxon) ? Status.OK_STATUS : Status.CANCEL_STATUS;
	}

	private boolean makeTaxonSynonym(Taxon taxon, Taxon destinationTaxon){
		// Move taxon in CDM to new destinationTaxon
		TaxonNameBase synonymName = taxon.getName();
		SynonymRelationshipType synonymType;
		if (CdmUtil.isNameHomotypic(synonymName, destinationTaxon)) {
			synonymType = SynonymRelationshipType.HOMOTYPIC_SYNONYM_OF();
		} else {
			synonymType = SynonymRelationshipType.HETEROTYPIC_SYNONYM_OF();
		}
		CdmUtil.makeTaxonSynonym(taxon, destinationTaxon, synonymType,
				null, null);
		
		// TODO move any children, descriptions, synonyms
		for (TaxonDescription description: taxon.getDescriptions()) {
			destinationTaxon.addDescription(description);
			taxon.removeDescription(description);
		}
		
		for (TaxonRelationship fromRelation : taxon.getRelationsFromThisTaxon()) {
			fromRelation.setFromTaxon(destinationTaxon);
		}
		
		for (TaxonRelationship toRelation : taxon.getRelationsToThisTaxon()) {
			toRelation.setToTaxon(destinationTaxon);
		}
				
		CdmSessionDataRepository.getDefault().removeTaxon(taxon);	
				
		// Close open editor for oldTaxon without forcing save.
		// User has already saved or declined to do so above.
		EditorController.close(taxon, false);
		CdmSessionDataRepository.getDefault().saveTaxon(destinationTaxon);
		// Open editor for destinationTaxon
		EditorController.open(destinationTaxon);
		
		return true;
	}
	
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return makeTaxonSynonym(taxon, destinationTaxon) ? Status.OK_STATUS : Status.CANCEL_STATUS;
	}

	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO implement undo
		throw new NotImplementedException();
	}
}
