/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetSorter;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.propertysheet.EditorPropertySheetEntry;
import eu.etaxonomy.taxeditor.propertysheet.PropertySheetUtil;

/**
 * The abstract editor for displaying a category of <code>Taxon</code> data, corresponding
 * to the tabs ("Name", "Descriptive", etc.) at the bottom of a <code>Taxon</code> view. Implements
 * <code>IAdaptable</code> in order to display properties of the objects whose UI elements have focus. 
 * <p>
 * Implementing classes can choose to show an object in the property sheet when the
 * <code>AbstractTaxonEditor</code> gets focus, by passing the object to the method
 * <code>setDefaultPropertySheetObject</code>, for instance, in the method<code>init</code>.
 * </p>
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 10.09.2008
 * @version 1.0
 */
public abstract class AbstractTaxonEditor extends EditorPart implements IAdaptable, IConversationEnabled, IPostOperationEnabled, ITabbedPropertySheetPageContributor  {
	private static final Logger logger = Logger
			.getLogger(AbstractTaxonEditor.class);

	protected Taxon taxon;
	
	/**
	 * When this <code>EditorPart</code> gets focus, the data structure of 
	 * <code>defaultPropertyObject</code> is displayed in the property sheet.
	 */
	
	protected ManagedForm managedForm;
	protected ScrolledForm scrolledForm; 
	protected Composite parent;
	protected ISelectionProvider provider;
	
	protected MultiPageTaxonEditor editor;
	
	protected IHasPropertySource selectedObject;

	protected Composite partComposite;
	protected GroupedComposite firstGroupedComposite;

	protected Object selectedData;
	
	protected ConversationHolder conversation;

	private MenuManager menuManager;

	private Menu menu;

	/**
	 * The object that was affected by last operation.
	 */
	private CdmBase objectAffectedByLastOperation;

	private boolean isRedrawing;
	
	protected AbstractTaxonEditor(MultiPageTaxonEditor editor){
		this.editor = editor;
		this.conversation = editor.getConversationHolder();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		conversation.commit();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	public void doSaveAs() {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
				"Invalid Input: Must be IEditorInput");
		
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			throw new PartInitException(
				"Invalid Input: Taxon cannot be null");
		}

		setSite(site);
		setInput(input);
		
		provider = new SimpleSelectionProvider();
		getSite().setSelectionProvider(provider);
		
		
		logger.trace("New Editor instance created.");
	}
	
	public IUndoContext getUndoContext() {
		return editor.getUndoContext();
	}
	
	/**
	 * If there is a default property sheet object with a corresponding property source class, 
	 * display it in the property sheet. Otherwise, empty the property sheet with an empty
	 * <code>StructuredSelection</code>.
	 * 
	 * @param selectedObject
	 */
	protected void setSelection(IHasPropertySource selectedObject) {
				
		// Unpaint last selection - last selection will only be unpainted
		//	when something else on this page is selected
		if (this.selectedObject instanceof GroupedComposite) {
			GroupedComposite composite = ((GroupedComposite) this.selectedObject);
			composite.unpaintBorder();
			composite.setBackground(Resources.getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
		}
		
		// Set the selection to this editor's selected object
		this.selectedObject = selectedObject;
		
		//  set background on selection TODO wanted to move this to setFocus() on the selectedObject but that doesn't work (infinite loop)
		((GroupedComposite) selectedObject).setBackground(Resources.getColor(Resources.COLOR_COMPOSITE_SELECTED));
		
		
		if (selectedObject instanceof Composite) {
			this.selectedData = ((Composite) selectedObject).getData();
		}
		
		// Get the selection's property source, pass it to the selection provider
		IPropertySource propertySource = null;
		if (selectedObject != null) {
			propertySource = selectedObject.getPropertySource();
		}
		if (propertySource == null) {
			provider.setSelection(new StructuredSelection());
		} else {
			provider.setSelection(new StructuredSelection(propertySource));
		}
		
//		// FIXME TabbedPropertySheet stuff 
//		provider.setSelection(new StructuredSelection(this.selectedObject));
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	public boolean isDirty() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite composite) {

		this.partComposite = composite;
		
		createManagedForm(composite);
	}
	
	protected void createManagedForm(Composite composite) {
		
		managedForm = new ManagedForm(composite) {
			public void dirtyStateChanged() {
				firePropertyChange(PROP_DIRTY);
			}
			public boolean setInput(Object input) {
				if (input instanceof IHasPropertySource) {
					setSelection((IHasPropertySource)input);
				}
				return super.setInput(input); 
			} 
		};
		
		scrolledForm = managedForm.getForm();
		parent = scrolledForm.getBody();
		
		// register the context menu
		menuManager = new MenuManager();
		ISelectionProvider selectionProvider = getSite().getSelectionProvider();
		getSite().registerContextMenu(getID(), menuManager, selectionProvider);
		
		menu = menuManager.createContextMenu(parent);
		scrolledForm.setMenu(menu);
				
		parent.setData(taxon);
		
		parent.setLayout(new TableWrapLayout());		
		parent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
	}
	
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class type) {
	    if (type == IPropertySheetPage.class) {
	    	
	    	PropertySheetPage page = new EditorPropertySheetPage();
	        
	        PropertySheetUtil.setPropertySheetPage(page);
	        
	        EditorPropertySheetEntry entry = new EditorPropertySheetEntry(taxon, this);
	        page.setRootEntry(entry);
	        page.refresh();
	        
	        return page;
	    	
//	        // FIXME TabbedPropertySheet stuff
//	    	EditorTabbedPropertySheetPage page = new EditorTabbedPropertySheetPage(this);
//	    	return page;	    	
	    }
	    
	    return super.getAdapter(type);
	}
	
	public Taxon getTaxon() {
		return taxon;
	}
	
	public void setDirty() {
		managedForm.dirtyStateChanged();
	}
	
	
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus(){
		Object selection;
		
		if(selectedData == null){
			selection = (objectAffectedByLastOperation == null) ? null : objectAffectedByLastOperation;
		}else{
			selection = (objectAffectedByLastOperation == null) ? selectedData : objectAffectedByLastOperation;
		}
				
		// make selection
		if (selection == null){
			GroupedComposite focusComposite = firstGroupedComposite;
			setSelection(focusComposite);
			focusComposite.drawBorder();
		} else {
			selectedData = selection;
			
			for (GroupedComposite composite : getGroupedComposites()) {
				if (selectedData.equals(composite.getData())) {
					setSelection((GroupedComposite) composite);
					((GroupedComposite) composite).drawBorder();
					((GroupedComposite) composite).setSelected();
					break;
				}					
			}
		}
		
		// reset
		objectAffectedByLastOperation = null;
	}
	
	/**
	 * This method returns all <code>GroupedComposite</code> controls currently attached to this editor.
	 * 
	 * @return a <code>List</code> of <code>GroupedComposite</code> controls.
	 */
	public List<GroupedComposite> getGroupedComposites(){
		return getGroupedCompositesRecursively(scrolledForm.getBody(), new ArrayList<GroupedComposite>());
	}

	
	/**
	 * Recursively traverse the composite hierarchy and collect all <code>GroupedComposite</code> controls.
	 * 
	 * @param composite the composite to start recursing from
	 * @param groupedComposites a <code>List</code> the found composites are stored in 
	 * @return a <code>List</code> containing all found <code>GroupedComposite</code> controls
	 */
	private List<GroupedComposite> getGroupedCompositesRecursively(Composite composite, List<GroupedComposite> groupedComposites){
		for(Control child : composite.getChildren()){
			if(child instanceof GroupedComposite){
				groupedComposites.add((GroupedComposite) child);
			}else if(child instanceof Composite){
				getGroupedCompositesRecursively((Composite) child, groupedComposites);
			}
		}
		return groupedComposites;
	}
	
	/**
	 * @return
	 */
	public List<GroupComposite> getGroupComposites() {
		return getGroupCompositesRecursively(scrolledForm.getBody(), new ArrayList<GroupComposite>());
	}
	
	/**
	 * Recursively traverse the composite hierarchy and collect all <code>GroupComposite</code> controls.
	 * 
	 * @param composite the composite to start recursing from
	 * @param groupComposites a <code>List</code> the found composites are stored in 
	 * @return a <code>List</code> containing all found <code>GroupComposite</code> controls
	 */
	private List<GroupComposite> getGroupCompositesRecursively(Composite composite, List<GroupComposite> groupComposites){
		for(Control child : composite.getChildren()){
			if(child instanceof GroupComposite){
				groupComposites.add((GroupComposite) child);
				getGroupCompositesRecursively((Composite) child, groupComposites);
			}
		}
		return groupComposites;
	}
	
	public void setInput(IEditorInput input){
		this.setInputWithNotify(input);		
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder(){
		return conversation;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostCrudObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	public void update(CdmDataChangeMap events) {
		//redraw();
	}
	
	/**
	 * Redraws this editor
	 * return true on success
	 */
	public boolean redraw(){
		return redraw(true);
	}
	
	/**
	 * Redraws the editor controls
	 * 
	 * @param focus if set to true the focus is set on the editor
	 * @return true on success
	 */
	public boolean redraw(boolean focus) {
		
		isRedrawing = true;
		
		this.selectedObject = null;
		managedForm.getForm().dispose();		
		createManagedForm(partComposite);
		
		setFocus();
		
		isRedrawing = false;
		
		return true;
		
	}
	
	public boolean getIsRedrawing() {
		return isRedrawing;
	}
	
	public MultiPageTaxonEditor getMultiPageTaxonEditor() {
		return editor;
	}


	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.operations.IPostOperationEnabled#postOperation()
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		
		if (isRedrawing) {
			return false;
		}
		
		if (objectAffectedByOperation == null && selectedData instanceof CdmBase) {
			this.objectAffectedByLastOperation = (CdmBase) selectedData;	
		} else {
			this.objectAffectedByLastOperation = objectAffectedByOperation;
		}
		editor.setDirty();
		return redraw();	
	}
	

	/**
	 * @return the managedForm
	 */
	public ManagedForm getManagedForm() {
		return managedForm;
	}
	
	public Composite getTopLevelComposite() {
		return this.getManagedForm().getForm().getBody();
	}
	
	/**
	 * 
	 * @return the ID as defined in plugin.xml
	 */
	public abstract String getID();

	/**
	 * @return the menu
	 */
	public Menu getMenu() {
		return menu;
	}

	/**
	 * @return the firstGroupedComposite
	 */
	public GroupedComposite getFirstGroupedComposite() {
		return firstGroupedComposite;
	}

	class EditorPropertySheetPage extends PropertySheetPage {
		
		EditorPropertySheetPage() {
			super();
			
			// Override sorter to simply display names as first-in-first-out
			setSorter(new PropertySheetSorter() {
				public int compare(IPropertySheetEntry entryA, IPropertySheetEntry entryB) {
					return 0;
				}
				public int compareCategories(String categoryA, String categoryB) {
					return 0;
				}
				public void sort(IPropertySheetEntry[] entries) {
					// do nothing
				}
			});
		}
		
        public void makeContributions(IMenuManager menuManager,
                IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
        	super.makeContributions(menuManager, toolBarManager, statusLineManager);
        
        	// Remove "Show categories", "Show advanced properties", "Restore default value"
        	toolBarManager.removeAll();
        	menuManager.removeAll();
        }
        
        public Control getControl() {
        	Control control = super.getControl();
        	
        	// Save the property sheet tree for easy access as needed
        	if (!control.isDisposed()) {
        		if (control instanceof Tree) {
        			PropertySheetUtil.setPropertySheetTree((Tree) control);
        		}
        	}
        	return control;
        }
	}
	
	/**
	 * 
	 */
	public void navigateToNextMember() {
		try{
			List<GroupedComposite> composites = getGroupedCompositesRecursively(scrolledForm.getBody(), new ArrayList<GroupedComposite>());
			
			for(int i = 0; i < composites.size(); i++){
				if(selectedData == composites.get(i).getData()){
					selectedData = composites.get(i+1).getData();
					break;
				}
			}
			setFocus();
		}catch(IndexOutOfBoundsException e){}
	}
	
	/**
	 * 
	 */
	public void navigateToPreviousMember() {
		try{
			List<GroupedComposite> composites = getGroupedCompositesRecursively(scrolledForm.getBody(), new ArrayList<GroupedComposite>());
			
			for(int i = 0; i < composites.size(); i++){
				if(selectedData == composites.get(i).getData()){
					selectedData = composites.get(i-1).getData();
					break;
				}
			}
			setFocus();
		}catch(IndexOutOfBoundsException e){}
	}
	
	/**
	 * Used by ITabbedPropertySheet...
	 */
    public String getContributorId() {
    	String id = getID();
        return id;
    }

}
