/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;

import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.model.AbstractUtility;

/**
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class EditorUtil extends AbstractUtility{
	private static final Logger logger = Logger.getLogger(EditorUtil.class);

	private static boolean isSaving = false;

	private static IStatusLineManager statusLineManager;

	private static IViewPart view;
	
	public static boolean closeAll() {
		for (IEditorPart editor : getOpenEditors()) {
			if (!getActivePage().closeEditor(editor, true)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Close the given editor.
	 * 
	 * @param editor The <tt>MultipageTaxonEditor</tt> to close.
	 * @return <tt>true</tt> on success
	 */
	public static boolean close(MultiPageTaxonEditor editor) {
		return getActivePage().closeEditor(editor, true);
	}

	/**
	 * Opens a new editor window with the given input
	 * 
	 * @param input
	 * @param editorId
	 * @return 
	 * @throws PartInitException
	 */
	private static IEditorPart open(IEditorInput input, String editorId)
			throws PartInitException {
		return getActivePage().openEditor(input, editorId);
	}
	

	/**
	 * Taxon Editors may be opened by supplying a taxon uuid only.
	 * Session gets initialised here and is passed to the editor
	 * 
	 * @param taxonNodeUuid
	 * @return
	 * @throws PartInitException 
	 */
	public static IEditorPart open(UUID taxonNodeUuid) throws PartInitException {
		IEditorInput input = TaxonEditorInput.NewInstance(taxonNodeUuid); 
		return open(input, MultiPageTaxonEditor.ID);
	}

	/**
	 * An uninitialized taxon is one that hasn't been saved yet. As such, it should appear in neither
	 * the list of recent names nor in the taxonomic tree when opened.
	 * 
	 * @param parentTaxon 	The UUID of the parental taxon or <code>null</code> if this is
	 * 						supposed to be a root taxon.
	 * @return 				The EditorPart.
	 * @throws PartInitException 
	 */
	public static IEditorPart openEmpty(UUID parentNodeUuid) throws PartInitException{
		TaxonEditorInput input = TaxonEditorInput.NewEmptyInstance(parentNodeUuid);
		return open(input, MultiPageTaxonEditor.ID);
	}

	public static void setSaving(boolean isSaving) {
		EditorUtil.isSaving = isSaving;
	}

	public static boolean isSaving() {
		return isSaving;
	}
	
	/**
	 * Returns a set of all currently open 
	 * <code>MultiPageTaxonEditor</code>s.
	 * 
	 * @return
	 */
	public static Set<IEditorPart> getOpenEditors() {
		Set<IEditorPart> taxonEditors = new HashSet<IEditorPart>();
		
		if(getActivePage() != null){
			for (IEditorReference reference : getActivePage().getEditorReferences()) {
				IEditorPart editor = reference.getEditor(false);
				if (editor instanceof MultiPageTaxonEditor) {
					taxonEditors.add(editor);
				}
			}
		}
		
		return taxonEditors;
	}
	
	/**
	 * Returns the currently active taxon editor
	 * 
	 * @return the taxon editor that has focus
	 */
	public static MultiPageTaxonEditor getActiveEditor(){
		IEditorPart editorPart = getActivePage().getActiveEditor();
		if(editorPart instanceof MultiPageTaxonEditor){
			MultiPageTaxonEditor editor = (MultiPageTaxonEditor) editorPart;
			editor.getConversationHolder().bind();
			return editor;
		}
		return null;
	}
	
	public static AbstractTaxonEditor getActiveEditorPage(Page page){
		MultiPageTaxonEditor editor = getActiveEditor();
		
		return editor.getPage(page);
	}
	
	public static IOperationHistory getOperationHistory() {
		return TaxeditorEditorPlugin.getDefault().getWorkbench().
					getOperationSupport().getOperationHistory();
	}
	
	public static IStatus executeOperation(IUndoableOperation operation){
		try {
			IStatus status = getOperationHistory().execute(operation, getMonitor(),
								WorkspaceUndoUtil.getUIInfoAdapter(getShell()));
			String statusString = status.equals(Status.OK_STATUS) ? "completed" : "cancelled";
			setStatusLine(operation.getLabel() + " " + statusString + ".");
			return status;
		} catch (ExecutionException e) {
			logger.error("Error executing operation: " + operation.getLabel(), e);
		}
		return null;
	}

	public static IUndoContext getUndoContext(MultiPageTaxonEditor editor){
		return editor.getUndoContext();
	}
	
	public static IUndoContext getUndoContext() {
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}

	private static IProgressMonitor getMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}
	
	public static void setStatusLineManager(IStatusLineManager manager) {
		statusLineManager = manager;
	}

	public static void setStatusLine(String message) {
		statusLineManager.setMessage(message);
	}

	public static void showPropertySheet() {
		if (getActivePage() == null) {
			return;
		}
		try {
			view = getActivePage().showView(IPageLayout.ID_PROP_SHEET);
		} catch (PartInitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static void checkHidePropertySheet() {
		if (getOpenEditors().size() == 0) {
			if (view != null && getActivePage() != null) {
				getActivePage().hideView(view);
			}
		}
	}
}
