/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.Set;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;

/**
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 18.03.2009
 * @version 1.0
 */
public class NameUtil {	

	@SuppressWarnings("unchecked")
	public static String getDisplayName(TaxonNameBase name){
		return name.getTitleCache();
	}
	
	public static String getDisplayName(TaxonBase<?> taxon){
		return getDisplayName(taxon.getName());
	}
	
	@SuppressWarnings("unchecked")
	public static String getDisplayNameWithRef(TaxonBase taxonBase) {
		TaxonNameBase name = taxonBase.getName();
		if (name != null) {
			if (name.getFullTitleCache() == null || name.getFullTitleCache().length() == 0) {
				return CdmUtils.Nz(name.getTitleCache());
			} else {
				return name.getFullTitleCache();
			}
		}
		return "";
	}
	
	/**
	 * Checks whether name is the basionym for ALL names 
	 * 	in its group.
	 * @param name
	 * 
	 * @return
	 */	
	@SuppressWarnings("unchecked")
	public static boolean isNameGroupBasionym(TaxonNameBase name) {
		if (name == null) {
			return false;
		}
		
		HomotypicalGroup homotypicalGroup = name.getHomotypicalGroup();
		if (homotypicalGroup == null) {
			return false;
		}		
		
		Set<TaxonNameBase> typifiedNames = homotypicalGroup.getTypifiedNames();
		
		// Check whether there are any other names in the group
		if (typifiedNames.size() == 1) {
			return false;
		}
		
		for (TaxonNameBase taxonName : typifiedNames) {
			if (!taxonName.equals(name)) {
				if (!isNameBasionymOf(name, taxonName)) {
					return false;
				}
			}
		}
		return true;		
	}
	
	/**
	 * Checks whether a basionym relationship exists between fromName and toName.
	 * 
	 * @param fromName
	 * @param toName
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static boolean isNameBasionymOf(TaxonNameBase fromName, TaxonNameBase toName) {
		Set<NameRelationship> relations = toName.getRelationsToThisName();
		for (NameRelationship relation : relations) {
			if (relation.getType().equals(NameRelationshipType.BASIONYM()) &&
					relation.getFromName().equals(fromName)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @param name
	 * @return
	 */
	public static boolean isNameSupraSpecific(TaxonNameBase<?, ?> name) {
		
		if (name == null || name.getRank() == null) {
			return false;
		}
		
//		Rank rank = CdmBase.deproxy(name.getRank(), Rank.class);
		Rank rank = name.getRank();
		
		if (rank.isHigher(Rank.SPECIES())) {
//		if (name.getRank().isHigher(Rank.SPECIES())) {
//		if (name.isInfraGeneric() || name.isSupraGeneric() || name.isGenus()) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Checks whether name belongs to the same homotypic group as taxon's name.
	 * 
	 * @param name
	 * @param taxon
	 * @return
	 */
	public static boolean isNameHomotypic(TaxonNameBase<?, ?> name, Taxon taxon) {
		TaxonNameBase<?, ?> taxonName = taxon.getName();
		if (taxonName == null || name == null) {
			return false;
		}
		HomotypicalGroup homotypicGroup = taxonName.getHomotypicalGroup();
		if (homotypicGroup == null) {
			return false;
		}
		if (homotypicGroup.equals(name.getHomotypicalGroup())) {
			return true;
		}
		return false;
	}
	
}
