/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.SourceViewerDecorationSupport;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 */
public class PreferencesUtil {
	private static final Logger logger = Logger
			.getLogger(PreferencesUtil.class);


	private static final String FEATURE_PREFERENCE = "FEATURE_PREFERENCE";


	private static SourceViewerDecorationSupport decorationSupport;

	/**
	 * 
	 * @return
	 */
	public static IPreferenceStore getPrefStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}
	
	/**
	 * 
	 * @param preferredCode
	 */
	public static void setNomenclaturalCode(String preferredCode) {
		getPrefStore().setValue(Resources.CODE_PREFERENCE, preferredCode);
	}
	
	/**
	 * 
	 */
	public static void setDefaultNomenclaturalCode() {
		setNomenclaturalCode(Resources.DEFAULT_CODE_PREFERENCE);
	}

	/**
	 * 
	 * @return
	 */
	public static String getPreferredNomenclaturalCodeAsString() {
		String nameCodePreference = getNameCodePreference();
		
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return "International Code of Botanical Nomenclature (ICBN)";
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		}
		return null;
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		String nameCodePreference = getNameCodePreference();
				
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return NomenclaturalCode.ICBN;
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return NomenclaturalCode.ICZN;
		}
		return null;
	}
	
	/**
	 * 
	 * @return
	 */
	private static String getNameCodePreference() {
		try{
			return getPrefStore().getString(Resources.CODE_PREFERENCE);
		}catch(NullPointerException e){
			logger.warn("PreferenceStore was not available. That is OK if you are running a unit test. Setting ICBN as default code.");
			return Resources.CODE_PREFERENCE_ICBN;
		}
	}

	/**
	 * Returns a <code>Set</code> of the <code>Feature</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Feature</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static List<Feature> getPreferredFeatures() {
		
		ArrayList<Feature> preferedFeatures = new ArrayList<Feature>();
		
		for (Feature feature : CdmStore.getFeatures()) {
			if(getPrefStore().getBoolean(getPreferenceKey(feature))){
				preferedFeatures.add(feature);
			}
		}
		
		return preferedFeatures;
	}

	/**
	 * 
	 * @param preferredFeatureList
	 */
	public static void setPreferredFeatures(List<Feature> preferredFeatureList) {
		
		for (Feature feature : CdmStore.getFeatures()) {
			getPrefStore().setValue(getPreferenceKey(feature), preferredFeatureList.contains(feature));
		}
	}
	
	/**
	 * Construct a unique key using <code>Feature</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Feature feature) {
		return FEATURE_PREFERENCE 
				. concat(".")
				. concat(feature.getUuid().toString());
	}

	/**
	 * Returns a <code>Set</code> of the <code>Rank</code>s that the user has chosen
	 * to have shown in preferences.
	 * </p>
	 * <p>
	 * <code>Rank</code>s are shown unless otherwise specified.
	 * </p>
	 * @return
	 */
	public static List<Rank> getPreferredRanks() {
		
		ArrayList<Rank> preferedRanks = new ArrayList<Rank>();
		
		for (Rank rank : CdmStore.getRanks()) {
			if(getPrefStore().getBoolean(getPreferenceKey(rank))){
				preferedRanks.add(rank);
			}
		}
		
		return preferedRanks;
	}
	
	/**
	 * 
	 * @param preferedRankList
	 */
	public static void setPreferredRanks(List<Rank> preferedRankList) {
		for (Rank rank : CdmStore.getRanks()) {
			getPrefStore().setValue(getPreferenceKey(rank), preferedRankList.contains(rank));
		}
	}



	/**
	 * Construct a unique key using <code>Rank</code>'s <code>Uuid</code>
	 * 
	 * @param feature
	 * @return
	 */
	private static String getPreferenceKey(Rank rank) {
		return Resources.RANK_PREFERENCE 
				. concat(".")
				. concat(rank.getUuid().toString());
	}

	/**
	 * Returns a new name object that conforms to the nomenclatural
	 * code specified in user preferences. 
	 * 
	 * @return
	 */
	public static NonViralName<?> getInstanceOfPreferredNameClass() {
		NomenclaturalCode code = getPreferredNomenclaturalCode();
		
		// Check whether name code preference needs to be initialized
		if (code == null) {
			setDefaultNomenclaturalCode();
		}
		
		if (code.equals(NomenclaturalCode.ICBN)) {
			return BotanicalName.NewInstance(null);
		} else if (code.equals(NomenclaturalCode.ICZN)) {
			return ZoologicalName.NewInstance(null);			
		}
		return NonViralName.NewInstance(null);
	}

	/**
	 * Appends the code (i.e. ICBN or ICZN) specified in the user preferences
	 * to a string. The result is separated by a period: "<code>key.ICBN</code>".
	 * 
	 * @param key
	 * @return
	 */
	public static String concatCodeMessageSuffix(String key) {
		String code = getNameCodePreference();
		return key + "." + code;
	}

	/**
	 * 
	 * @return
	 */
	public static SourceViewerDecorationSupport getDecorationSupport() {
		if (decorationSupport == null) {
			decorationSupport = new SourceViewerDecorationSupport(null, null, null, EditorsUI.getSharedTextColors());
		}
		return decorationSupport;
	}
}
